@@ -19,7 +19,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.10.18"
+__version__ = "0.10.20"
 
 # Used with pypi checks and other messages related to pip
 _wandb_module = "wandb"
@@ -97,13 +97,26 @@ from wandb.integration.sagemaker import sagemaker_auth
 _IS_INTERNAL_PROCESS = False
 
 
-def _set_internal_process():
+def _set_internal_process(disable=False):
     global _IS_INTERNAL_PROCESS
+    if _IS_INTERNAL_PROCESS is None:
+        return
+    if disable:
+        _IS_INTERNAL_PROCESS = None
+        return
     _IS_INTERNAL_PROCESS = True
 
 
-def _is_internal_process():
-    return _IS_INTERNAL_PROCESS
+def _assert_is_internal_process():
+    if _IS_INTERNAL_PROCESS is None:
+        return
+    assert _IS_INTERNAL_PROCESS
+
+
+def _assert_is_user_process():
+    if _IS_INTERNAL_PROCESS is None:
+        return
+    assert not _IS_INTERNAL_PROCESS
 
 
 # toplevel:
@@ -132,6 +145,9 @@ use_artifact = _preinit.PreInitCallable(
 log_artifact = _preinit.PreInitCallable(
     "wandb.log_artifact", wandb_sdk.wandb_run.Run.log_artifact
 )
+_define_metric = _preinit.PreInitCallable(
+    "wandb._define_metric", wandb_sdk.wandb_run.Run._define_metric
+)
 plot_table = _preinit.PreInitCallable(
     "wandb.plot_table", wandb_sdk.wandb_run.Run.plot_table
 )
