@@ -4,11 +4,11 @@ import six
 from six.moves import queue
 import time
 
-from watchdog.observers.polling import PollingObserver
-from watchdog.events import PatternMatchingEventHandler
 from wandb import util
 import glob
 
+wd_polling = util.vendor_import("watchdog.observers.polling")
+wd_events = util.vendor_import("watchdog.events")
 
 logger = logging.getLogger(__file__)
 
@@ -157,7 +157,7 @@ class DirWatcher(object):
         self._user_file_policies = {"end": set(), "live": set(), "now": set()}
         self._file_pusher = file_pusher
         self._file_event_handlers = {}
-        self._file_observer = PollingObserver()
+        self._file_observer = wd_polling.PollingObserver()
         self._file_observer.schedule(
             self._per_file_event_handler(), self._dir, recursive=True
         )
@@ -189,7 +189,7 @@ class DirWatcher(object):
     def _per_file_event_handler(self):
         """Create a Watchdog file event handler that does different things for every file
         """
-        file_event_handler = PatternMatchingEventHandler()
+        file_event_handler = wd_events.PatternMatchingEventHandler()
         file_event_handler.on_created = self._on_file_created
         file_event_handler.on_modified = self._on_file_modified
         file_event_handler.on_moved = self._on_file_moved
