@@ -5,6 +5,7 @@ module sdk
 """
 
 from . import wandb_helper as helper  # noqa: F401
+from .wandb_alerts import AlertLevel  # noqa: F401
 from .wandb_artifacts import Artifact  # noqa: F401
 from .wandb_config import Config  # noqa: F401
 from .wandb_history import History  # noqa: F401
@@ -15,4 +16,4 @@ from .wandb_save import save  # noqa: F401
 from .wandb_settings import Settings  # noqa: F401
 from .wandb_setup import setup  # noqa: F401
 from .wandb_summary import Summary  # noqa: F401
-from .wandb_watch import watch  # noqa: F401
+from .wandb_watch import unwatch, watch  # noqa: F401
