@@ -15,7 +15,8 @@ import six
 from six.moves import queue
 import wandb
 from wandb import data_types
-from wandb.proto import wandb_internal_pb2  # type: ignore
+from wandb.proto import wandb_internal_pb2 as pb
+from wandb.proto import wandb_telemetry_pb2 as tpb
 from wandb.util import (
     get_h5_typename,
     json_dumps_safer,
@@ -25,53 +26,76 @@ from wandb.util import (
     WandBJSONEncoderOld,
 )
 
-if wandb.TYPE_CHECKING:  # type: ignore
+from .artifacts import ArtifactManifest
+from ..wandb_artifacts import Artifact
+
+if wandb.TYPE_CHECKING:
     import typing as t
     from . import summary_record as sr
-    from typing import Optional
-    from wandb.proto.wandb_internal_pb2 import PollExitResponse
+    from typing import Any, Dict, Iterable, Optional, Tuple, Union
+    from multiprocessing import Process
+    from typing import cast
+    from typing import TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        from ..wandb_run import Run
+        from six.moves.queue import Queue
+else:
+
+    def cast(_, val):
+        return val
+
 
 logger = logging.getLogger("wandb")
 
 
-def file_policy_to_enum(policy):
+def file_policy_to_enum(policy: str) -> "pb.FilesItem.PolicyTypeValue":
     if policy == "now":
-        enum = wandb_internal_pb2.FilesItem.PolicyType.NOW
+        enum = pb.FilesItem.PolicyType.NOW
     elif policy == "end":
-        enum = wandb_internal_pb2.FilesItem.PolicyType.END
+        enum = pb.FilesItem.PolicyType.END
     elif policy == "live":
-        enum = wandb_internal_pb2.FilesItem.PolicyType.LIVE
+        enum = pb.FilesItem.PolicyType.LIVE
     return enum
 
 
-def file_enum_to_policy(enum):
-    if enum == wandb_internal_pb2.FilesItem.PolicyType.NOW:
+def file_enum_to_policy(enum: "pb.FilesItem.PolicyTypeValue") -> str:
+    if enum == pb.FilesItem.PolicyType.NOW:
         policy = "now"
-    elif enum == wandb_internal_pb2.FilesItem.PolicyType.END:
+    elif enum == pb.FilesItem.PolicyType.END:
         policy = "end"
-    elif enum == wandb_internal_pb2.FilesItem.PolicyType.LIVE:
+    elif enum == pb.FilesItem.PolicyType.LIVE:
         policy = "live"
     return policy
 
 
+class _Future(object):
+    _object: Optional[pb.Result]
+
+    def __init__(self) -> None:
+        self._object = None
+        self._object_ready = threading.Event()
+        self._lock = threading.Lock()
+
+    def get(self, timeout: int = None) -> Optional[pb.Result]:
+        is_set = self._object_ready.wait(timeout)
+        if is_set and self._object:
+            return self._object
+        return None
+
+    def _set_object(self, obj: pb.Result) -> None:
+        self._object = obj
+        self._object_ready.set()
+
+
 class MessageRouter(object):
-    class _Future(object):
-        def __init__(self):
-            self._object = None
-            self._object_ready = threading.Event()
-            self._lock = threading.Lock()
+    _pending_reqs: Dict[str, _Future]
+    _request_queue: "Queue[pb.Record]"
+    _response_queue: "Queue[pb.Result]"
 
-        def get(self, timeout=None) -> Optional[object]:
-            is_set = self._object_ready.wait(timeout)
-            if is_set and self._object:
-                return self._object
-            return None
-
-        def _set_object(self, obj):
-            self._object = obj
-            self._object_ready.set()
-
-    def __init__(self, request_queue, response_queue):
+    def __init__(
+        self, request_queue: "Queue[pb.Record]", response_queue: "Queue[pb.Result]"
+    ) -> None:
         self._request_queue = request_queue
         self._response_queue = response_queue
 
@@ -83,7 +107,7 @@ class MessageRouter(object):
         self._thread.daemon = True
         self._thread.start()
 
-    def message_loop(self):
+    def message_loop(self) -> None:
         while not self._join_event.is_set():
             try:
                 msg = self._response_queue.get(timeout=1)
@@ -91,12 +115,12 @@ class MessageRouter(object):
                 continue
             self._handle_msg_rcv(msg)
 
-    def send_and_receive(self, rec, local=None):
+    def send_and_receive(self, rec: pb.Record, local: Optional[bool] = None) -> _Future:
         rec.control.req_resp = True
         if local:
             rec.control.local = local
         rec.uuid = uuid.uuid4().hex
-        future = self._Future()
+        future = _Future()
         with self._lock:
             self._pending_reqs[rec.uuid] = future
 
@@ -104,11 +128,11 @@ class MessageRouter(object):
 
         return future
 
-    def join(self):
+    def join(self) -> None:
         self._join_event.set()
         self._thread.join()
 
-    def _handle_msg_rcv(self, msg):
+    def _handle_msg_rcv(self, msg: pb.Result) -> None:
         with self._lock:
             future = self._pending_reqs.pop(msg.uuid, None)
         if future is None:
@@ -121,9 +145,18 @@ class BackendSender(object):
     class ExceptionTimeout(Exception):
         pass
 
+    record_q: Optional["Queue[pb.Record]"]
+    result_q: Optional["Queue[pb.Result]"]
+    process: Optional[Process]
+    _run: Optional["Run"]
+    _router: Optional[MessageRouter]
+
     def __init__(
-        self, record_q=None, result_q=None, process=None,
-    ):
+        self,
+        record_q: "Queue[pb.Record]" = None,
+        result_q: "Queue[pb.Result]" = None,
+        process: Process = None,
+    ) -> None:
         self.record_q = record_q
         self.result_q = result_q
         self._process = process
@@ -133,72 +166,78 @@ class BackendSender(object):
         if record_q and result_q:
             self._router = MessageRouter(record_q, result_q)
 
-    def _hack_set_run(self, run):
+    def _hack_set_run(self, run: "Run") -> None:
         self._run = run
 
-    def publish_output(self, name, data):
+    def publish_output(self, name: str, data: str) -> None:
         # from vendor.protobuf import google3.protobuf.timestamp
         # ts = timestamp.Timestamp()
         # ts.GetCurrentTime()
         # now = datetime.now()
         if name == "stdout":
-            otype = wandb_internal_pb2.OutputRecord.OutputType.STDOUT
+            otype = pb.OutputRecord.OutputType.STDOUT
         elif name == "stderr":
-            otype = wandb_internal_pb2.OutputRecord.OutputType.STDERR
+            otype = pb.OutputRecord.OutputType.STDERR
         else:
             # TODO(jhr): throw error?
             print("unknown type")
-        o = wandb_internal_pb2.OutputRecord(output_type=otype, line=data)
+        o = pb.OutputRecord(output_type=otype, line=data)
         o.timestamp.GetCurrentTime()
         self._publish_output(o)
 
-    def _publish_output(self, outdata):
-        rec = wandb_internal_pb2.Record()
+    def _publish_output(self, outdata: pb.OutputRecord) -> None:
+        rec = pb.Record()
         rec.output.CopyFrom(outdata)
         self._publish(rec)
 
-    def publish_tbdata(self, log_dir, save, root_logdir):
-        tbrecord = wandb_internal_pb2.TBRecord()
+    def publish_tbdata(
+        self, log_dir: str, save: bool, root_logdir: Optional[str]
+    ) -> None:
+        tbrecord = pb.TBRecord()
         tbrecord.log_dir = log_dir
         tbrecord.save = save
         tbrecord.root_dir = root_logdir or ""
         rec = self._make_record(tbrecord=tbrecord)
         self._publish(rec)
 
-    def _publish_history(self, history):
+    def _publish_history(self, history: pb.HistoryRecord) -> None:
         rec = self._make_record(history=history)
         self._publish(rec)
 
-    def publish_history(self, data, step=None, run=None, publish_step=True):
+    def publish_history(
+        self, data: dict, step: int = None, run: "Run" = None, publish_step: bool = True
+    ) -> None:
         run = run or self._run
         data = data_types.history_dict_to_json(run, data, step=step)
-        history = wandb_internal_pb2.HistoryRecord()
+        history = pb.HistoryRecord()
         if publish_step:
+            assert step is not None
             history.step.num = step
         data.pop("_step", None)
         for k, v in six.iteritems(data):
             item = history.item.add()
             item.key = k
-            item.value_json = json_dumps_safer_history(v)
+            item.value_json = json_dumps_safer_history(v)  # type: ignore
         self._publish_history(history)
 
-    def publish_telemetry(self, telem):
+    def publish_telemetry(self, telem: tpb.TelemetryRecord) -> None:
         rec = self._make_record(telemetry=telem)
         self._publish(rec)
 
-    def _make_run(self, run):
-        proto_run = wandb_internal_pb2.RunRecord()
+    def _make_run(self, run: "Run") -> pb.RunRecord:
+        proto_run = pb.RunRecord()
         run._make_proto_run(proto_run)
-        proto_run.host = run._settings.host
+        if run._settings.host:
+            proto_run.host = run._settings.host
         if run._config is not None:
-            config_dict = run._config._as_dict()
+            config_dict = run._config._as_dict()  # type: ignore
             self._make_config(data=config_dict, obj=proto_run.config)
         if run._telemetry_obj:
             proto_run.telemetry.MergeFrom(run._telemetry_obj)
         return proto_run
 
-    def _make_artifact(self, artifact):
-        proto_artifact = wandb_internal_pb2.ArtifactRecord()
+    def _make_artifact(self, artifact: Artifact) -> pb.ArtifactRecord:
+        proto_artifact = pb.ArtifactRecord()
         proto_artifact.type = artifact.type
         proto_artifact.name = artifact.name
         proto_artifact.digest = artifact.digest
@@ -211,9 +250,11 @@ class BackendSender(object):
         self._make_artifact_manifest(artifact.manifest, obj=proto_artifact.manifest)
         return proto_artifact
 
-    def _make_artifact_manifest(self, artifact_manifest, obj=None):
-        proto_manifest = obj or wandb_internal_pb2.ArtifactManifest()
-        proto_manifest.version = artifact_manifest.version()
+    def _make_artifact_manifest(
+        self, artifact_manifest: ArtifactManifest, obj: pb.ArtifactManifest = None
+    ) -> pb.ArtifactManifest:
+        proto_manifest = obj or pb.ArtifactManifest()
+        proto_manifest.version = artifact_manifest.version()  # type: ignore
         proto_manifest.storage_policy = artifact_manifest.storage_policy.name()
 
         for k, v in artifact_manifest.storage_policy.config().items() or {}.items():
@@ -221,7 +262,7 @@ class BackendSender(object):
             cfg.key = k
             cfg.value_json = json.dumps(v)
 
-        for entry in sorted(artifact_manifest.entries.values(), key=lambda k: k.path):
+        for entry in sorted(artifact_manifest.entries.values(), key=lambda k: k.path):  # type: ignore
             proto_entry = proto_manifest.contents.add()
             proto_entry.path = entry.path
             proto_entry.digest = entry.digest
@@ -238,38 +279,45 @@ class BackendSender(object):
                 proto_extra.value_json = json.dumps(v)
         return proto_manifest
 
-    def _make_exit(self, exit_code):
-        exit = wandb_internal_pb2.RunExitRecord()
+    def _make_exit(self, exit_code: int) -> pb.RunExitRecord:
+        exit = pb.RunExitRecord()
         exit.exit_code = exit_code
         return exit
 
-    def _make_config(self, data=None, key=None, val=None, obj=None):
-        config = obj or wandb_internal_pb2.ConfigRecord()
+    def _make_config(
+        self,
+        data: dict = None,
+        key: Union[Tuple[str, ...], str] = None,
+        val: Any = None,
+        obj: pb.ConfigRecord = None,
+    ) -> pb.ConfigRecord:
+        config = obj or pb.ConfigRecord()
         if data:
             for k, v in six.iteritems(data):
                 update = config.update.add()
                 update.key = k
-                update.value_json = json_dumps_safer(json_friendly(v)[0])
+                update.value_json = json_dumps_safer(json_friendly(v)[0])  # type: ignore
         if key:
             update = config.update.add()
             if isinstance(key, tuple):
-                update.nested_key = key
+                for k in key:
+                    update.nested_key.append(k)
             else:
                 update.key = key
-            update.value_json = json_dumps_safer(json_friendly(val)[0])
+            update.value_json = json_dumps_safer(json_friendly(val)[0])  # type: ignore
         return config
 
-    def _make_stats(self, stats_dict):
-        stats = wandb_internal_pb2.StatsRecord()
-        stats.stats_type = wandb_internal_pb2.StatsRecord.StatsType.SYSTEM
+    def _make_stats(self, stats_dict: dict) -> pb.StatsRecord:
+        stats = pb.StatsRecord()
+        stats.stats_type = pb.StatsRecord.StatsType.SYSTEM
         stats.timestamp.GetCurrentTime()
         for k, v in six.iteritems(stats_dict):
             item = stats.item.add()
             item.key = k
-            item.value_json = json_dumps_safer(json_friendly(v)[0])
+            item.value_json = json_dumps_safer(json_friendly(v)[0])  # type: ignore
         return stats
 
-    def _summary_encode(self, value: t.Any, path_from_root: str):
+    def _summary_encode(self, value: t.Any, path_from_root: str) -> dict:
         """Normalize, compress, and encode sub-objects for backend storage.
 
         value: Object to encode.
@@ -292,13 +340,13 @@ class BackendSender(object):
                 )
             return json_value
         else:
-            friendly_value, converted = json_friendly(
+            friendly_value, converted = json_friendly(  # type: ignore
                 data_types.val_to_json(
                     self._run, path_from_root, value, namespace="summary"
                 )
             )
-            json_value, compressed = maybe_compress_summary(
-                friendly_value, get_h5_typename(value)
+            json_value, compressed = maybe_compress_summary(  # type: ignore
+                friendly_value, get_h5_typename(value)  # type: ignore
             )
             if compressed:
                 # TODO(jhr): impleement me
@@ -307,16 +355,16 @@ class BackendSender(object):
 
             return json_value
 
-    def _make_summary_from_dict(self, summary_dict):
-        summary = wandb_internal_pb2.SummaryRecord()
+    def _make_summary_from_dict(self, summary_dict: dict) -> pb.SummaryRecord:
+        summary = pb.SummaryRecord()
         for k, v in six.iteritems(summary_dict):
             update = summary.update.add()
             update.key = k
             update.value_json = json.dumps(v)
         return summary
 
-    def _make_summary(self, summary_record: sr.SummaryRecord):
-        pb_summary_record = wandb_internal_pb2.SummaryRecord()
+    def _make_summary(self, summary_record: sr.SummaryRecord) -> pb.SummaryRecord:
+        pb_summary_record = pb.SummaryRecord()
 
         for item in summary_record.update:
             pb_summary_item = pb_summary_record.update.add()
@@ -331,7 +379,7 @@ class BackendSender(object):
 
             path_from_root = ".".join(item.key)
             json_value = self._summary_encode(item.value, path_from_root)
-            json_value, _ = json_friendly(json_value)
+            json_value, _ = json_friendly(json_value)  # type: ignore
 
             pb_summary_item.value_json = json.dumps(
                 json_value, cls=WandBJSONEncoderOld,
@@ -350,35 +398,34 @@ class BackendSender(object):
 
         return pb_summary_record
 
-    def _make_files(self, files_dict):
-        files = wandb_internal_pb2.FilesRecord()
+    def _make_files(self, files_dict: dict) -> pb.FilesRecord:
+        files = pb.FilesRecord()
         for path, policy in files_dict["files"]:
             f = files.files.add()
             f.path = path
             f.policy = file_policy_to_enum(policy)
         return files
 
-    def _make_login(self, api_key=None, anonymous=None):
-        login = wandb_internal_pb2.LoginRequest()
+    def _make_login(self, api_key: str = None) -> pb.LoginRequest:
+        login = pb.LoginRequest()
         if api_key:
             login.api_key = api_key
-        if anonymous:
-            login.anonymous = anonymous
         return login
 
     def _make_request(
         self,
-        login=None,
-        get_summary=None,
-        pause=None,
-        resume=None,
-        status=None,
-        poll_exit=None,
-        sampled_history=None,
-        run_start=None,
-        check_version=None,
-    ):
-        request = wandb_internal_pb2.Request()
+        login: pb.LoginRequest = None,
+        get_summary: pb.GetSummaryRequest = None,
+        pause: pb.PauseRequest = None,
+        resume: pb.ResumeRequest = None,
+        status: pb.StatusRequest = None,
+        poll_exit: pb.PollExitRequest = None,
+        sampled_history: pb.SampledHistoryRequest = None,
+        run_start: pb.RunStartRequest = None,
+        check_version: pb.CheckVersionRequest = None,
+        defer: pb.DeferRequest = None,
+    ) -> pb.Record:
+        request = pb.Request()
         if login:
             request.login.CopyFrom(login)
         elif get_summary:
@@ -397,6 +444,8 @@ class BackendSender(object):
             request.run_start.CopyFrom(run_start)
         elif check_version:
             request.check_version.CopyFrom(check_version)
+        elif defer:
+            request.defer.CopyFrom(defer)
         else:
             raise Exception("Invalid request")
         record = self._make_record(request=request)
@@ -406,23 +455,24 @@ class BackendSender(object):
 
     def _make_record(
         self,
-        run=None,
-        config=None,
-        files=None,
-        summary=None,
-        history=None,
-        stats=None,
-        exit=None,
-        artifact=None,
-        tbrecord=None,
-        alert=None,
-        final=None,
-        header=None,
-        footer=None,
-        request=None,
-        telemetry=None,
-    ):
-        record = wandb_internal_pb2.Record()
+        run: pb.RunRecord = None,
+        config: pb.ConfigRecord = None,
+        files: pb.FilesRecord = None,
+        summary: pb.SummaryRecord = None,
+        history: pb.HistoryRecord = None,
+        stats: pb.StatsRecord = None,
+        exit: pb.RunExitRecord = None,
+        artifact: pb.ArtifactRecord = None,
+        tbrecord: pb.TBRecord = None,
+        alert: pb.AlertRecord = None,
+        final: pb.FinalRecord = None,
+        metric: pb.MetricRecord = None,
+        header: pb.HeaderRecord = None,
+        footer: pb.FooterRecord = None,
+        request: pb.Request = None,
+        telemetry: tpb.TelemetryRecord = None,
+    ) -> pb.Record:
+        record = pb.Record()
         if run:
             record.run.CopyFrom(run)
         elif config:
@@ -453,25 +503,32 @@ class BackendSender(object):
             record.request.CopyFrom(request)
         elif telemetry:
             record.telemetry.CopyFrom(telemetry)
+        elif metric:
+            record.metric.CopyFrom(metric)
         else:
             raise Exception("Invalid record")
         return record
 
-    def _publish(self, record, local=None):
+    def _publish(self, record: pb.Record, local: bool = None) -> None:
         if self._process and not self._process.is_alive():
             raise Exception("The wandb backend process has shutdown")
         if local:
             record.control.local = local
-        self.record_q.put(record)
+        if self.record_q:
+            self.record_q.put(record)
 
-    def _communicate(self, rec, timeout=5, local=None):
+    def _communicate(
+        self, rec: pb.Record, timeout: Optional[int] = 5, local: bool = None
+    ) -> Optional[pb.Result]:
         assert self._router
         future = self._router.send_and_receive(rec, local=local)
         f = future.get(timeout)
         return f
 
-    def communicate_login(self, api_key=None, anonymous=None, timeout=15):
-        login = self._make_login(api_key, anonymous)
+    def communicate_login(
+        self, api_key: str = None, timeout: Optional[int] = 15
+    ) -> pb.LoginResponse:
+        login = self._make_login(api_key)
         rec = self._make_request(login=login)
         result = self._communicate(rec, timeout=timeout)
         if result is None:
@@ -483,66 +540,80 @@ class BackendSender(object):
         assert login_response
         return login_response
 
-    def publish_defer(self, state=0):
-        rec = wandb_internal_pb2.Record()
-        rec.request.defer.CopyFrom(wandb_internal_pb2.DeferRequest(state=state))
+    def _publish_defer(self, state: "pb.DeferRequest.DeferStateValue") -> None:
+        defer = pb.DeferRequest(state=state)
+        rec = self._make_request(defer=defer)
         self._publish(rec, local=True)
 
-    def publish_header(self):
-        header = wandb_internal_pb2.HeaderRecord()
+    def publish_defer(self, state: int = 0) -> None:
+        self._publish_defer(cast("pb.DeferRequest.DeferStateValue", state))
+
+    def publish_header(self) -> None:
+        header = pb.HeaderRecord()
         rec = self._make_record(header=header)
         self._publish(rec)
 
-    def publish_footer(self):
-        footer = wandb_internal_pb2.FooterRecord()
+    def publish_footer(self) -> None:
+        footer = pb.FooterRecord()
         rec = self._make_record(footer=footer)
         self._publish(rec)
 
-    def publish_final(self):
-        final = wandb_internal_pb2.FinalRecord()
+    def publish_final(self) -> None:
+        final = pb.FinalRecord()
         rec = self._make_record(final=final)
         self._publish(rec)
 
-    def publish_login(self, api_key=None, anonymous=None):
-        login = self._make_login(api_key, anonymous)
+    def publish_login(self, api_key: str = None) -> None:
+        login = self._make_login(api_key)
         rec = self._make_request(login=login)
         self._publish(rec)
 
-    def publish_pause(self):
-        pause = wandb_internal_pb2.PauseRequest()
+    def publish_pause(self) -> None:
+        pause = pb.PauseRequest()
         rec = self._make_request(pause=pause)
         self._publish(rec)
 
-    def publish_resume(self):
-        resume = wandb_internal_pb2.ResumeRequest()
+    def publish_resume(self) -> None:
+        resume = pb.ResumeRequest()
         rec = self._make_request(resume=resume)
         self._publish(rec)
 
-    def _publish_run(self, run):
+    def _publish_run(self, run: pb.RunRecord) -> None:
         rec = self._make_record(run=run)
         self._publish(rec)
 
-    def publish_run(self, run_obj):
+    def publish_run(self, run_obj: "Run") -> None:
         run = self._make_run(run_obj)
         self._publish_run(run)
 
-    def publish_config(self, data=None, key=None, val=None):
+    def publish_config(
+        self,
+        data: dict = None,
+        key: Union[Tuple[str, ...], str] = None,
+        val: Any = None,
+    ) -> None:
         cfg = self._make_config(data=data, key=key, val=val)
         self._publish_config(cfg)
 
-    def _publish_config(self, cfg):
+    def _publish_config(self, cfg: pb.ConfigRecord) -> None:
         rec = self._make_record(config=cfg)
         self._publish(rec)
 
-    def publish_summary(self, summary_record: sr.SummaryRecord):
+    def publish_summary(self, summary_record: sr.SummaryRecord) -> None:
         pb_summary_record = self._make_summary(summary_record)
         self._publish_summary(pb_summary_record)
 
-    def _publish_summary(self, summary):
+    def _publish_summary(self, summary: pb.SummaryRecord) -> None:
         rec = self._make_record(summary=summary)
         self._publish(rec)
 
-    def _communicate_run(self, run, timeout=None):
+    def _publish_metric(self, metric: pb.MetricRecord) -> None:
+        rec = self._make_record(metric=metric)
+        self._publish(rec)
+
+    def _communicate_run(
+        self, run: pb.RunRecord, timeout: int = None
+    ) -> Optional[pb.RunUpdateResult]:
         """Send synchronous run object waiting for a response.
 
         Arguments:
@@ -558,33 +629,35 @@ class BackendSender(object):
         if resp is None:
             logger.info("couldn't get run from backend")
             # Note: timeouts handled by callers: wandb_init.py
-            return
-        assert resp.run_result
+            return None
+        assert resp.HasField("run_result")
         return resp.run_result
 
-    def communicate_run(self, run_obj, timeout=None):
+    def communicate_run(
+        self, run_obj: "Run", timeout: int = None
+    ) -> Optional[pb.RunUpdateResult]:
         run = self._make_run(run_obj)
         return self._communicate_run(run, timeout=timeout)
 
-    def publish_stats(self, stats_dict):
+    def publish_stats(self, stats_dict: dict) -> None:
         stats = self._make_stats(stats_dict)
         rec = self._make_record(stats=stats)
         self._publish(rec)
 
-    def publish_files(self, files_dict):
+    def publish_files(self, files_dict: dict) -> None:
         files = self._make_files(files_dict)
         rec = self._make_record(files=files)
         self._publish(rec)
 
     def publish_artifact(
         self,
-        run,
-        artifact,
-        aliases,
-        is_user_created=False,
-        use_after_commit=False,
-        finalize=True,
-    ):
+        run: "Run",
+        artifact: Artifact,
+        aliases: Iterable[str],
+        is_user_created: bool = False,
+        use_after_commit: bool = False,
+        finalize: bool = True,
+    ) -> None:
         proto_run = self._make_run(run)
         proto_artifact = self._make_artifact(artifact)
         proto_artifact.run_id = proto_run.run_id
@@ -598,8 +671,10 @@ class BackendSender(object):
         rec = self._make_record(artifact=proto_artifact)
         self._publish(rec)
 
-    def publish_alert(self, title, text, level, wait_duration):
-        proto_alert = wandb_internal_pb2.AlertRecord()
+    def publish_alert(
+        self, title: str, text: str, level: str, wait_duration: int
+    ) -> None:
+        proto_alert = pb.AlertRecord()
         proto_alert.title = title
         proto_alert.text = text
         proto_alert.level = level
@@ -607,21 +682,27 @@ class BackendSender(object):
         rec = self._make_record(alert=proto_alert)
         self._publish(rec)
 
-    def communicate_status(self, check_stop_req, timeout=None):
-        status = wandb_internal_pb2.StatusRequest()
+    def communicate_status(
+        self, check_stop_req: bool, timeout: int = None
+    ) -> Optional[pb.StatusResponse]:
+        status = pb.StatusRequest()
         status.check_stop_req = check_stop_req
         req = self._make_request(status=status)
 
         resp = self._communicate(req, timeout=timeout, local=True)
+        if resp is None:
+            return None
         assert resp.response.status_response
         return resp.response.status_response
 
-    def publish_exit(self, exit_code):
+    def publish_exit(self, exit_code: int) -> None:
         exit_data = self._make_exit(exit_code)
         rec = self._make_record(exit=exit_data)
         self._publish(rec)
 
-    def _communicate_exit(self, exit_data, timeout=None):
+    def _communicate_exit(
+        self, exit_data: pb.RunExitRecord, timeout: int = None
+    ) -> pb.RunExitResult:
         req = self._make_record(exit=exit_data)
 
         result = self._communicate(req, timeout=timeout)
@@ -633,8 +714,10 @@ class BackendSender(object):
         assert result.exit_result
         return result.exit_result
 
-    def communicate_poll_exit(self, timeout=None) -> Optional[PollExitResponse]:
-        poll_request = wandb_internal_pb2.PollExitRequest()
+    def communicate_poll_exit(
+        self, timeout: int = None
+    ) -> Optional[pb.PollExitResponse]:
+        poll_request = pb.PollExitRequest()
         rec = self._make_request(poll_exit=poll_request)
         result = self._communicate(rec, timeout=timeout)
         if result is None:
@@ -643,49 +726,51 @@ class BackendSender(object):
         assert poll_exit_response
         return poll_exit_response
 
-    def communicate_check_version(self, current_version=None):
-        check_version = wandb_internal_pb2.CheckVersionRequest()
+    def communicate_check_version(
+        self, current_version: str = None
+    ) -> Optional[pb.CheckVersionResponse]:
+        check_version = pb.CheckVersionRequest()
         if current_version:
             check_version.current_version = current_version
         rec = self._make_request(check_version=check_version)
         result = self._communicate(rec)
         if result is None:
             # Note: timeouts handled by callers: wandb_init.py
-            return
+            return None
         return result.response.check_version_response
 
-    def communicate_run_start(self, run_pb):
-        run_start = wandb_internal_pb2.RunStartRequest()
+    def communicate_run_start(self, run_pb: pb.RunRecord) -> Optional[pb.Result]:
+        run_start = pb.RunStartRequest()
         run_start.run.CopyFrom(run_pb)
         rec = self._make_request(run_start=run_start)
         result = self._communicate(rec)
         return result
 
-    def communicate_exit(self, exit_code, timeout=None):
+    def communicate_exit(self, exit_code: int, timeout: int = None) -> pb.RunExitResult:
         exit_data = self._make_exit(exit_code)
         return self._communicate_exit(exit_data, timeout=timeout)
 
-    def communicate_summary(self):
-        record = self._make_request(get_summary=wandb_internal_pb2.GetSummaryRequest())
+    def communicate_summary(self) -> Optional[pb.GetSummaryResponse]:
+        record = self._make_request(get_summary=pb.GetSummaryRequest())
         result = self._communicate(record, timeout=10)
+        if result is None:
+            return None
         get_summary_response = result.response.get_summary_response
         assert get_summary_response
         return get_summary_response
 
-    def communicate_sampled_history(self):
-        record = self._make_request(
-            sampled_history=wandb_internal_pb2.SampledHistoryRequest()
-        )
+    def communicate_sampled_history(self) -> Optional[pb.SampledHistoryResponse]:
+        record = self._make_request(sampled_history=pb.SampledHistoryRequest())
         result = self._communicate(record)
+        if result is None:
+            return None
         sampled_history_response = result.response.sampled_history_response
         assert sampled_history_response
         return sampled_history_response
 
-    def join(self):
+    def join(self) -> None:
         # shutdown
-        request = wandb_internal_pb2.Request(
-            shutdown=wandb_internal_pb2.ShutdownRequest()
-        )
+        request = pb.Request(shutdown=pb.ShutdownRequest())
         record = self._make_record(request=request)
         _ = self._communicate(record)
 
